@@ -13,7 +13,6 @@
  *	David Dawes, Andrew E. Mileski, Leonard N. Zubkoff,
  *	Guy DESBIEF, Itai Nahshon.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/cirrus/lg_driver.c,v 1.49 2003/11/03 05:11:09 tsi Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -29,9 +28,6 @@
 
 #include "compiler.h"
 
-/* Drivers for PCI hardware need this */
-#include "xf86PciInfo.h"
-
 /* Drivers that need to access the PCI config space directly need this */
 #include "xf86Pci.h"
 
@@ -39,12 +35,17 @@
 /* This driver needs to be modified to not use vgaHW for multihead operation */
 #include "vgaHW.h"
 
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 6
 #include "xf86RAC.h"
 #include "xf86Resources.h"
+#endif
 
 /* All drivers initialising the SW cursor need this */
 #include "mipointer.h"
 
+/* need this for inputInfo */
+#include "inputstr.h"
+
 /* All drivers implementing backing store need this */
 #include "mibstore.h"
 
@@ -74,20 +75,20 @@
 
 /* Mandatory functions */
 Bool LgPreInit(ScrnInfoPtr pScrn, int flags);
-Bool LgScreenInit(int Index, ScreenPtr pScreen, int argc, char **argv);
-Bool LgEnterVT(int scrnIndex, int flags);
-void LgLeaveVT(int scrnIndex, int flags);
-static Bool	LgCloseScreen(int scrnIndex, ScreenPtr pScreen);
+Bool LgScreenInit(SCREEN_INIT_ARGS_DECL);
+Bool LgEnterVT(VT_FUNC_ARGS_DECL);
+void LgLeaveVT(VT_FUNC_ARGS_DECL);
+static Bool	LgCloseScreen(CLOSE_SCREEN_ARGS_DECL);
 static Bool	LgSaveScreen(ScreenPtr pScreen, Bool mode);
 
 /* Required if the driver supports mode switching */
-Bool LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags);
+Bool LgSwitchMode(SWITCH_MODE_ARGS_DECL);
 /* Required if the driver supports moving the viewport */
-void LgAdjustFrame(int scrnIndex, int x, int y, int flags);
+void LgAdjustFrame(ADJUST_FRAME_ARGS_DECL);
 
 /* Optional functions */
-void LgFreeScreen(int scrnIndex, int flags);
-ModeStatus LgValidMode(int scrnIndex, DisplayModePtr mode,
+void LgFreeScreen(FREE_SCREEN_ARGS_DECL);
+ModeStatus LgValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode,
 		       Bool verbose, int flags);
 
 /* Internally used functions */
@@ -161,80 +162,6 @@ static int LgLinePitches[4][11] = {
 	/* 32 */ { 160,  256,  320,  416,  512,  640,  832, 1024, 1280, 1664, 0 }
 };
 
-/*
- * List of symbols from other modules that this module references.  This
- * list is used to tell the loader that it is OK for symbols here to be
- * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
- * unresolved symbols that are not required.
- */
-
-static const char *vgahwSymbols[] = {
-	"vgaHWFreeHWRec",
-	"vgaHWGetHWRec",
-	"vgaHWGetIOBase",
-	"vgaHWGetIndex",
-	"vgaHWHandleColormaps",
-	"vgaHWInit",
-	"vgaHWLock",
-	"vgaHWMapMem",
-	"vgaHWProtect",
-	"vgaHWRestore",
-	"vgaHWSave",
-	"vgaHWSaveScreen",
-	"vgaHWUnlock",
-	NULL
-};
-
-static const char *fbSymbols[] = {
-	"fbScreenInit",
-	"fbPictureInit",
-	NULL
-};
-
-static const char *xaaSymbols[] = {
-	"XAACreateInfoRec",
-	"XAADestroyInfoRec",
-	"XAAInit",
-	NULL
-};
-
-static const char *ramdacSymbols[] = {
-	"xf86CreateCursorInfoRec",
-	"xf86DestroyCursorInfoRec",
-	"xf86InitCursor",
-	NULL
-};
-
-#define LGuseI2C 1
-
-static const char *ddcSymbols[] = {
-	"xf86PrintEDID",
-#if LGuseI2C
-	"xf86DoEDID_DDC2",
-#endif
-	"xf86SetDDCproperties",
-	NULL
-};
-
-static const char *i2cSymbols[] = {
-	"xf86CreateI2CBusRec",
-	"xf86I2CBusInit",
-	NULL
-};
-
-static const char *int10Symbols[] = {
-	"xf86FreeInt10",
-	"xf86InitInt10",
-	NULL
-};
-
-static const char *shadowSymbols[] = {
-    "ShadowFBInit",
-    NULL
-};
-
 #ifdef XFree86LOADER
 
 #define LG_MAJOR_VERSION 1
@@ -274,9 +201,6 @@ lgSetup(pointer module, pointer opts, int *errmaj, int *errmin)
     
     if (!setupDone) {
 	setupDone = TRUE;
-	LoaderRefSymLists(vgahwSymbols, fbSymbols, xaaSymbols,
-			  ramdacSymbols, ddcSymbols, i2cSymbols,
-			  int10Symbols, NULL);
     }
     return (pointer)1;
 }
@@ -331,7 +255,7 @@ LgFreeRec(ScrnInfoPtr pScrn)
 {
 	if (pScrn->driverPrivate == NULL)
 		return;
-	xfree(pScrn->driverPrivate);
+	free(pScrn->driverPrivate);
 	pScrn->driverPrivate = NULL;
 }
 
@@ -430,8 +354,6 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 	if (!xf86LoadSubModule(pScrn, "vgahw"))
 		return FALSE;
 
-	xf86LoaderReqSymLists(vgahwSymbols, NULL);
-
 	/*
 	 * Allocate a vgaHWRec
 	 */
@@ -439,6 +361,7 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 		return FALSE;
 
 	hwp = VGAHWPTR(pScrn);
+	vgaHWSetStdFuncs(hwp);
 	vgaHWGetIOBase(hwp);
 
 	/* Allocate the LgRec driverPrivate */
@@ -447,7 +370,12 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 
 	pCir = CIRPTR(pScrn);
 	pCir->pScrn = pScrn;
+
+#if GET_ABI_MAJOR(ABI_VIDEODRV_VERSION) < 12
 	pCir->PIOReg = hwp->PIOOffset + 0x3CE;
+#else
+	pCir->PIOReg = 0x3CE;
+#endif
 
 	/* Get the entity, and make sure it is PCI. */
 	pCir->pEnt = xf86GetEntityInfo(pScrn->entityList[0]);
@@ -463,7 +391,6 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 
 	if (xf86LoadSubModule(pScrn, "int10")) {
 	    xf86Int10InfoPtr int10InfoPtr;
-	    xf86LoaderReqSymLists(int10Symbols, NULL);
 	    
 	    int10InfoPtr = xf86InitInt10(pCir->pEnt->index);
 
@@ -529,7 +456,7 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 	xf86CollectOptions(pScrn, NULL);
 
 	/* Process the options */
-	if (!(pCir->Options = xalloc(sizeof(LgOptions))))
+	if (!(pCir->Options = malloc(sizeof(LgOptions))))
 		return FALSE;
 	memcpy(pCir->Options, LgOptions, sizeof(LgOptions));
 	xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pCir->Options);
@@ -648,6 +575,7 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 	pCir->FbMapSize = pScrn->videoRam * 1024;
 	pCir->IoMapSize = 0x4000;	/* 16K for moment,  will increase */
 
+#ifndef XSERVER_LIBPCIACCESS
 	pScrn->racIoFlags =   RAC_COLORMAP 
 #ifndef EXPERIMENTAL
 	  | RAC_VIEWPORT
@@ -661,19 +589,17 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 			"xf86RegisterResources() found resource conflicts\n");
 		return FALSE;
 	}
-
+#endif
 	if (!xf86LoadSubModule(pScrn, "ddc")) {
 		LgFreeRec(pScrn);
 		return FALSE;
 	}
-	xf86LoaderReqSymLists(ddcSymbols, NULL);
 
 #if LGuseI2C
 	if (!xf86LoadSubModule(pScrn, "i2c")) {
 		LgFreeRec(pScrn);
 		return FALSE;
 	}
-	xf86LoaderReqSymLists(i2cSymbols, NULL);
 #endif
 
 	/* Read and print the monitor DDC information */
@@ -866,17 +792,17 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 	         LgFreeRec(pScrn);
 		 return FALSE;
 	    }
-	    xf86LoaderReqSymLists(fbSymbols, NULL);
 	    break;
 	}
 
 	/* Load XAA if needed */
 	if (!pCir->NoAccel) {
 		if (!xf86LoadSubModule(pScrn, "xaa")) {
-			LgFreeRec(pScrn);
-			return FALSE;
+			xf86DrvMsg(pScrn->scrnIndex, X_INFO,
+				   "Falling back to shadowfb\n");
+			pCir->NoAccel = TRUE;
+			pCir->shadowFB = TRUE;
 		}
-		xf86LoaderReqSymLists(xaaSymbols, NULL);
 	}
 
 	/* Load ramdac if needed */
@@ -885,7 +811,6 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 			LgFreeRec(pScrn);
 			return FALSE;
 		}
-		xf86LoaderReqSymLists(ramdacSymbols, NULL);
 	}
 
 	if (pCir->shadowFB) {
@@ -893,7 +818,6 @@ LgPreInit(ScrnInfoPtr pScrn, int flags)
 		LgFreeRec(pScrn);
 		return FALSE;
 	    }
-	    xf86LoaderReqSymLists(shadowSymbols, NULL);
 	}
 	
 	return TRUE;
@@ -1305,10 +1229,10 @@ LgRestore(ScrnInfoPtr pScrn)
 /* This gets called at the start of each server generation */
 
 Bool
-LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+LgScreenInit(SCREEN_INIT_ARGS_DECL)
 {
 	/* The vgaHW references will disappear one day */
-	ScrnInfoPtr pScrn;
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	vgaHWPtr hwp;
 	CirPtr pCir;
 	int i, ret;
@@ -1320,11 +1244,6 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	ErrorF("LgScreenInit\n");
 #endif
 
-	/*
-	 * First get the ScrnInfoRec
-	 */
-	pScrn = xf86Screens[pScreen->myNum];
-
 	hwp = VGAHWPTR(pScrn);
 
 	hwp->MapSize = 0x10000;		/* Standard 64k VGA window */
@@ -1354,7 +1273,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	LgSaveScreen(pScreen, SCREEN_SAVER_ON);
 
 	/* Set the viewport */
-	LgAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
+	LgAdjustFrame(ADJUST_FRAME_ARGS(pScrn, pScrn->frameX0, pScrn->frameY0));
 
 	/*
 	 * The next step is to setup the screen's visuals, and initialise the
@@ -1394,7 +1313,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	
 	if(pCir->shadowFB) {
 	    pCir->ShadowPitch = BitmapBytePad(pScrn->bitsPerPixel * width);
-	    pCir->ShadowPtr = xalloc(pCir->ShadowPitch * height);
+	    pCir->ShadowPtr = malloc(pCir->ShadowPitch * height);
 	    displayWidth = pCir->ShadowPitch / (pScrn->bitsPerPixel >> 3);
 	    FbBase = pCir->ShadowPtr;
 	} else {
@@ -1417,7 +1336,7 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 				displayWidth,pScrn->bitsPerPixel);
 	    break;
 	default:
-		xf86DrvMsg(scrnIndex, X_ERROR,
+		xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 			   "X11: Internal error: invalid bpp (%d) in LgScreenInit\n",
 			   pScrn->bitsPerPixel);
 		ret = FALSE;
@@ -1516,9 +1435,10 @@ LgScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
 /* Usually mandatory */
 Bool
-LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
+LgSwitchMode(SWITCH_MODE_ARGS_DECL)
 {
-	return LgModeInit(xf86Screens[scrnIndex], mode);
+	SCRN_INFO_PTR(arg);
+	return LgModeInit(pScrn, mode);
 }
 
 #define ROUND_DOWN(x, mod)	(((x) / (mod)) * (mod))
@@ -1530,9 +1450,9 @@ LgSwitchMode(int scrnIndex, DisplayModePtr mode, int flags)
  */
 /* Usually mandatory */
 void
-LgAdjustFrame(int scrnIndex, int x, int y, int flags)
+LgAdjustFrame(ADJUST_FRAME_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	int Base, tmp;
 	CirPtr pCir = CIRPTR(pScrn);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
@@ -1618,9 +1538,9 @@ LgAdjustFrame(int scrnIndex, int x, int y, int flags)
 
 /* Mandatory */
 Bool
-LgEnterVT(int scrnIndex, int flags)
+LgEnterVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	CirPtr pCir = CIRPTR(pScrn);
 #ifdef LG_DEBUG
 	ErrorF("LgEnterVT\n");
@@ -1645,9 +1565,9 @@ LgEnterVT(int scrnIndex, int flags)
 
 /* Mandatory */
 void
-LgLeaveVT(int scrnIndex, int flags)
+LgLeaveVT(VT_FUNC_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	SCRN_INFO_PTR(arg);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 	CirPtr pCir = CIRPTR(pScrn);
 #ifdef LG_DEBUG
@@ -1673,9 +1593,9 @@ LgLeaveVT(int scrnIndex, int flags)
 
 /* Mandatory */
 static Bool
-LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
+LgCloseScreen(CLOSE_SCREEN_ARGS_DECL)
 {
-	ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
+	ScrnInfoPtr pScrn = xf86ScreenToScrn(pScreen);
 	vgaHWPtr hwp = VGAHWPTR(pScrn);
 	CirPtr pCir = CIRPTR(pScrn);
 
@@ -1697,14 +1617,14 @@ LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
 		xf86DestroyCursorInfoRec(pCir->CursorInfoRec);
 	pCir->CursorInfoRec = NULL;
 	if (pCir->DGAModes)
-		xfree(pCir->DGAModes);
+		free(pCir->DGAModes);
 	pCir->DGAnumModes = 0;
 	pCir->DGAModes = NULL;
 
 	pScrn->vtSema = FALSE;
 
 	pScreen->CloseScreen = pCir->CloseScreen;
-	return (*pScreen->CloseScreen)(scrnIndex, pScreen);
+	return (*pScreen->CloseScreen)(CLOSE_SCREEN_ARGS);
 }
 
 
@@ -1712,8 +1632,9 @@ LgCloseScreen(int scrnIndex, ScreenPtr pScreen)
 
 /* Optional */
 void
-LgFreeScreen(int scrnIndex, int flags)
+LgFreeScreen(FREE_SCREEN_ARGS_DECL)
 {
+	SCRN_INFO_PTR(arg);
 #ifdef LG_DEBUG
 	ErrorF("LgFreeScreen\n");
 #endif
@@ -1722,8 +1643,8 @@ LgFreeScreen(int scrnIndex, int flags)
 	 * get called routinely at the end of a server generation.
 	 */
 	if (xf86LoaderCheckSymbol("vgaHWFreeHWRec"))
-		vgaHWFreeHWRec(xf86Screens[scrnIndex]);
-	LgFreeRec(xf86Screens[scrnIndex]);
+		vgaHWFreeHWRec(pScrn);
+	LgFreeRec(pScrn);
 }
 
 
@@ -1731,7 +1652,7 @@ LgFreeScreen(int scrnIndex, int flags)
 
 /* Optional */
 ModeStatus
-LgValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
+LgValidMode(SCRN_ARG_TYPE arg, DisplayModePtr mode, Bool verbose, int flags)
 {
 	int lace;
 
@@ -1757,14 +1678,14 @@ LgValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 static Bool
 LgSaveScreen(ScreenPtr pScreen, int mode)
 {
-	CirPtr pCir = CIRPTR(xf86Screens[pScreen->myNum]);
+	CirPtr pCir = CIRPTR(xf86ScreenToScrn(pScreen));
 	ScrnInfoPtr pScrn = NULL;
 	Bool unblank;
 
 	unblank = xf86IsUnblank(mode);
 
 	if (pScreen != NULL)
-	    pScrn = xf86Screens[pScreen->myNum];
+	    pScrn = xf86ScreenToScrn(pScreen);
 
 	if (pScrn != NULL && pScrn->vtSema) {
 	    if (unblank)
